#include "stm32f10x.h"

#define GPIO_TYPE_IN_FLOATING			0x4
#define GPIO_TYPE_IN_PULL_UP			0x8
#define GPIO_TYPE_OUT_GEN_PURP_PUSH_PULL	0x1

#define GPIO_VALUE_INPUT_PULL_DOWN		0x0
#define GPIO_VALUE_INPUT_PULL_UP		0x1

#define STACK_TOP 0x20002000

void nmi_handler(void);												
void hardfault_handler(void);
void delay(void);
int main(void);
void _start(void);

extern unsigned long _etext;
extern unsigned long _sidata;		/* start address for the initialization values of the .data section. defined in linker script */
extern unsigned long _sdata;		/* start address for the .data section. defined in linker script */
extern unsigned long _edata;		/* end address for the .data section. defined in linker script */

extern unsigned long _sbss;			/* start address for the .bss section. defined in linker script */
extern unsigned long _ebss;			/* end address for the .bss section. defined in linker script */

/*	Four vectors - the starting stack pointer value, code entry point and NMI and Hard-Fault handlers */

unsigned int * myvectors[4] 
__attribute__ ((section("vectors")))= {
	(unsigned int *)	STACK_TOP,         
	(unsigned int *) 	_start,              
	(unsigned int *)	nmi_handler,       
	(unsigned int *)	hardfault_handler  
};


void _start(void)
{
	unsigned long *pulDest;
	unsigned long *pulSrc;

	//
	// Copy the data segment initializers from flash to SRAM in ROM mode
	//

	if (&_sidata != &_sdata) {	// only if needed
		pulSrc = &_sidata;
		for(pulDest = &_sdata; pulDest < &_edata; ) {
			*(pulDest++) = *(pulSrc++);
		}
	}

	//
	// Zero fill the bss segment.
	//
	for(pulDest = &_sbss; pulDest < &_ebss; )
	{
		*(pulDest++) = 0;
	}

	main();
	while(1);
}

void nmi_handler(void)
{
	return;
}


void hardfault_handler(void)
{
	return;
}

#define APB2_USART1EN		(1 << 14)
#define APB2_IOPCEN		(1 << 4)
#define APB2_IOPBEN		(1 << 3)
#define APB2_IOPAEN		(1 << 2)
#define APB2_AFIOEN		(1 << 0)
#define APB1_TIM3EN		(1 << 1)
void rcc_init(void)
{
	//APB2 peripheral clock enable register
	RCC->APB2ENR |= (APB2_USART1EN | APB2_IOPCEN | APB2_IOPBEN | APB2_IOPAEN | APB2_AFIOEN);
	//RCC->APB1ENR |= (APB1_TIM3EN);
}

void gpio_set_type(GPIO_TypeDef *port, int pin, int value)
{
	if (pin < 8 && pin >= 0) {
		port->CRL |= (value << (pin * 4));
	} 
	else if (pin < 16 && pin >= 8)
	{
		port->CRH |= (value << ((pin - 8) * 4));
	}
	//else { error }
	
}

void gpio_set_value(GPIO_TypeDef *port, int pin, int value)
{
	if (value == 1 && (pin >= 0 && pin < 16)) {
		port->BSRR |= (1 << pin);
	}
	else if (value == 0 && (pin >= 0 && pin < 16)) {
		port->BSRR |= (1 << pin << 16);
	}
	//else { error }
}

int gpio_get_value(GPIO_TypeDef *port, int pin)
{
	if ((port->IDR & (1 << pin)) > 0) {
		return 1;
	}
	else {
		return 0;
	}

}

void gpio_init()
{
	GPIOA->CRL = 0x0;
	GPIOA->CRH = 0x0;
	GPIOB->CRL = 0x0;
	GPIOB->CRH = 0x0;
	GPIOC->CRL = 0x0;
	GPIOC->CRH = 0x0;
	GPIOD->CRL = 0x0;
	GPIOD->CRH = 0x0;
}
//----------------------------------------------------------------------------
void delay(void) 
{
	int i = 300000;

	for(i; i > 0; i--) {
		asm("nop");
	}
}

int main(void)
{
	rcc_init();	
	gpio_init();

	gpio_set_type(GPIOC, 8, GPIO_TYPE_OUT_GEN_PURP_PUSH_PULL);
	
	while(1) {
		gpio_set_value(GPIOC, 8, 1);
		delay();
		gpio_set_value(GPIOC, 8, 0);
		delay();
	}
	
}

